package dao;

import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import dto.Gender;
import dto.Pet;
import dto.PetType;

public class PetDAO {
	private PetDAO() { 
	}
	
	private static PetDAO instance = new PetDAO();
	public static PetDAO getInstance(){
		return instance;
	} 

	// insert
	public int insert(Connection conn, Pet pet) throws SQLException {
		PreparedStatement pstmt = null;
		try {
			String sql = "insert into pet values (?,?,?,?,?,?)";
			pstmt = conn.prepareStatement(sql);
			pstmt.setString(1, pet.getName());
			pstmt.setString(2, pet.getOwner());
			pstmt.setString(3, pet.getSpecies().toString());
			pstmt.setString(4, pet.getSex().toString());
			pstmt.setDate(5, pet.getBirth());
			pstmt.setDate(6, pet.getDeath());
			return pstmt.executeUpdate();
		} finally {
			if (pstmt != null) {
				pstmt.close();
			}
		}
	}

	// delete
	public int delete(Connection conn, String name) throws SQLException {
		PreparedStatement pstmt = null;
		try {
			String sql = "delete from pet where name=?";
			pstmt = conn.prepareStatement(sql);
			pstmt.setString(1, name);
			return pstmt.executeUpdate();
		} finally {
			if (pstmt != null) {
				pstmt.close();
			}
		}
	}

	// edit
	public int edit(Connection conn, Pet pet) throws SQLException {
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			String sql = "update pet set owner=?, species=?, sex=?, birth=?, death=? where name=?";
			pstmt = conn.prepareStatement(sql);
			pstmt.setString(1, pet.getOwner());
			pstmt.setString(2, pet.getSpecies().toString());
			pstmt.setString(3, pet.getSex().toString());
			pstmt.setDate(4, pet.getBirth());
			pstmt.setDate(5, pet.getDeath());
			pstmt.setString(6, pet.getName());
			return pstmt.executeUpdate();
		} finally {
			if (rs != null) {
				rs.close();
			}
			if (pstmt != null) {
				pstmt.close();
			}
		}
	}
	
	// select(find/get)
	public Pet select(Connection conn, String name) throws SQLException {
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			String sql = "select * from pet where name=?";
			pstmt = conn.prepareStatement(sql);
			pstmt.setString(1, name);
			rs = pstmt.executeQuery();
			if (rs.next()) {
				return createFromResultSet(rs);
			}
			else {
				return null;
			}
		} finally {
			if (rs != null) {
				rs.close();
			}
			if (pstmt != null) {
				pstmt.close();
			}
		}
	}
	
	public Pet createFromResultSet(ResultSet rs) throws SQLException {
		String name = rs.getString("name");
		String owner = rs.getString("owner");
		PetType species = PetType.nameOf(rs.getString("species"));
		Gender sex = Gender.nameOf(rs.getString("sex"));
		Date birth = rs.getDate("birth");
		Date death = rs.getDate("death");
		Pet pet = new Pet(name, owner, species, sex, birth, death);
		return pet;
	}
	
	// selectList
	public List<Pet> selectList(Connection conn) throws SQLException {
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			String sql = "select * from pet";
			pstmt = conn.prepareStatement(sql);
			rs = pstmt.executeQuery();
			List<Pet> petList = new ArrayList<>();
			while (rs.next()) {
				petList.add(createFromResultSet(rs));
			}
			return petList;
		} finally {
			if (rs != null) {
				rs.close();
			}
			if (pstmt != null) {
				pstmt.close();
			}
		}
	}
	
	// selectListByOwner
	public List<Pet> selectListByOwner(Connection conn, String owner) throws SQLException {
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			String sql = "select * from pet order by owner=?";
			pstmt = conn.prepareStatement(sql);
			pstmt.setString(1, owner);
			rs = pstmt.executeQuery();
			List<Pet> productList = new ArrayList<>();
			while (rs.next()) {
				productList.add(createFromResultSet(rs));
			}
			return productList;
		} finally {
			if (rs != null) {
				rs.close();
			}
			if (pstmt != null) {
				pstmt.close();
			}
		}
	}
}
